#	ZXMedia: ZX Spectrum file format manipulation library
#	Copyright (C) 2004 Matthew Westcott
#	
#	This file is part of ZXMedia.
#	
#	ZXMedia is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License version 2, as
#	published by the Free Software Foundation.
#	
#	ZXMedia is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with ZXMedia; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Contact address: Matthew Westcott, 14 Daisy Hill Drive, Adlington,
#	Chorley, Lancs. PR6 9NE, United Kingdom

module ZXMedia
	class BASICBlock < Block
		# This class represents a BASIC program block.
		
		attr_reader(:filename, :autostart_line, :basic_length, :data)
		attr_accessor(:tape_representation, :trdos_representation)

		def initialize(filename, data, basic_length, autostart_line = nil)
			@tape_representation = nil
			@trdos_representation = nil

			@filename = filename
			@data = data
			@basic_length = basic_length
			@autostart_line = autostart_line
		end

		def filename=(f)
			@filename = f
		end
		
		def autostart_line=(line)
			line = nil unless (1..9999) === line
			@autostart_line = line
		end

		def basic_length=(len)
			@basic_length = len
		end

		def block_length
			@data.size
		end

		def to_a
			@data
		end

		def trdos_block
			@trdos_representation = self.canonical_trdos_representation if (@trdos_representation.nil?)
			@trdos_representation.trdos_block
		end

		def write_tap_block(io)
			@tape_representation = self.canonical_tape_representation if (@tape_representation.nil?)
			@tape_representation.each { |block|
				block.write_tap_block(io)
			}
		end

		def write_tzx_block(io)
			@tape_representation = self.canonical_tape_representation if (@tape_representation.nil?)
			@tape_representation.each { |block|
				block.write_tzx_block(io)
			}
		end

		def canonical_tape_representation
			[ProgramHeaderTapeBlock.new(self), DataTapeBlock.new(self)]
		end

		def canonical_trdos_representation
			TRDOSBASICBlock.new(self, @filename, @basic_length, @autostart_line)
		end

		def BASICBlock.from_tape(subblocks)
			# find the header and datablock
			header = nil
			datablock = nil
			subblocks.each { |block|
				if block.is_a? ProgramHeaderTapeBlock
					header = block
				end
				if block.is_a? DataTapeBlock
					datablock = block
					break
				end unless header.nil?
			}
			raise "ProgramHeaderTapeBlock missing from tape block list" if header.nil?
			raise "DataTapeBlock missing from tape block list" if datablock.nil?
			
			basic_block = BASICBlock.new(header.filename, datablock.data, header.basic_length, header.autostart_line)
			basic_block.tape_representation = subblocks
			header.datasource = basic_block
			datablock.datasource = basic_block
			basic_block
		end

		def BASICBlock.from_trdos(trdos)
			raise "BASICBlock.from_trdos called on a non-TRDOSBASICBlock block" unless trdos.is_a? TRDOSBASICBlock
			basic_block = BASICBlock.new(trdos.filename, trdos.data, trdos.basic_length, trdos.autostart_line)
			basic_block.trdos_representation = trdos
			trdos.data = basic_block
			basic_block
		end

		def BASICBlock.can_promote?(block)
			block.is_a? TRDOSBASICBlock
		end

		def BASICBlock.promote(block)
			basic_block = BASICBlock.new(block.filename, block.data, block.basic_length, block.autostart_line)
			basic_block.trdos_representation = block
			block.datasource = basic_block
			basic_block
		end

		def to_s
			"Program: \"#{self.filename}\"" + (self.autostart_line.nil? ? '' : " LINE #{self.autostart_line}") + " (#{self.data.length} bytes)"
		end

	end # BASICBlock
end # ZXMedia