#	ZXMedia: ZX Spectrum file format manipulation library
#	Copyright (C) 2004 Matthew Westcott
#	
#	This file is part of ZXMedia.
#	
#	ZXMedia is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License version 2, as
#	published by the Free Software Foundation.
#	
#	ZXMedia is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with ZXMedia; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Contact address: Matthew Westcott, 14 Daisy Hill Drive, Adlington,
#	Chorley, Lancs. PR6 9NE, United Kingdom

module ZXMedia

	##
	# A Block is any unit of Spectrum data, meta-information or control code
	# which may be transferred from one Medium to another. It may be specific
	# to a certain media type, such as a PureToneTapeBlock, or generic like a
	# BASICBlock. Certain blocks may in turn be made up of lower-level blocks,
	# such as a BASICBlock which can be broken down into a ProgramHeaderTapeBlock
	# and a DataTapeBlock.

	class Block

		## 
		# Indicates whether this type of block can appear between a HeaderTapeBlock
		# and a DataTapeBlock on a tape without removing the ability to read those
		# two blocks as a single entity
		def preserves_tape_loading_continuity?
			false
		end

		##
		# Return the result of promoting this block to its highest-level form possible;
		# for example, a StandardSpeedTapeBlock might be promoted to a BytesHeaderTapeBlock
		# or a DataTapeBlock depending on its contents
		def rise
			block = self
			begin
				promotion_succeeds = false
				block.class.promotion_targets.each { |target|
					if target.can_promote?(block)
						block = target.promote(block)
						promotion_succeeds = true
						break
					end
				}
			end while promotion_succeeds
			block
		end

		#########
		protected
		#########

		# List of classes to which we might be able to promote this type of Block
		def Block.promotion_targets
			# none by default
			[]
		end

		# Determine whether the given candidate block can be promoted to this type of Block
		def Block.can_promote?(candidate)
			# nothing can be promoted to Block
			false
		end
		
	end # Block
end # ZXMedia