#	ZXMedia: ZX Spectrum file format manipulation library
#	Copyright (C) 2004 Matthew Westcott
#	
#	This file is part of ZXMedia.
#	
#	ZXMedia is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License version 2, as
#	published by the Free Software Foundation.
#	
#	ZXMedia is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with ZXMedia; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Contact address: Matthew Westcott, 14 Daisy Hill Drive, Adlington,
#	Chorley, Lancs. PR6 9NE, United Kingdom

module ZXMedia

	## 
	# A Medium is a representation of a tape, disk or other unit of storage
	# used on the Spectrum. It takes the form of a collection of [Block]s.
	class Medium

		# An array of the blocks contained within this medium
		attr_reader :blocks
		
		# A text label describing the medium; used for TR-DOS disk images
		attr_accessor :label

		##
		# Create a new empty medium

		def initialize
			@blocks = []
			@label = ''
		end

		##
		# Return a string description of the medium contents
		
		def to_s
			@blocks.join("\n")
		end

		##
		# Write this medium out to the supplied IO object, as a TAP tape image
		
		def write_tap(io)
			@blocks.each { |block|
				if block.respond_to?(:write_tap_block)
					block.write_tap_block(io)
				else
					$stderr << "Warning: Cannot write block #{block} to TAP format - skipping\n"
				end
			}
		end

		##
		# Write this medium out to the supplied IO object, as a TZX tape image

		def write_tzx(io)
			io << 'ZXTape!'
			io.putc 0x1a; io.putc 1; io.putc 13
			@blocks.each { |block|
				if block.respond_to?(:write_tzx_block)
					block.write_tzx_block(io)
				else
					$stderr << "Warning: Cannot write block #{block} to TZX format - skipping\n"
				end
			}
		end

		##
		# Write this medium out to the supplied IO object, as an SCL disk image

		def write_scl(io)
			sum_io = ChecksummedIO.new(io)
			writable_blocks = []
			@blocks.each { |block|
				if block.respond_to?(:trdos_block)
					writable_blocks << block.trdos_block
				else
					$stderr << "Warning: Cannot write block #{block} to SCL format - skipping\n"
				end
			}
			sum_io << 'SINCLAIR'
			sum_io.putc writable_blocks.size
			# write directory listing
			writable_blocks.each { |block|
				sector_count = block.sector_count
				sum_io << [block.filename, block.extension, block.param1, block.param2, sector_count].pack('a8avvc')
			}
			# write data
			writable_blocks.each { |block|
				data = block.data.to_a
				padding = [0] * (block.sector_count * 256 - data.size)
				sum_io << (data + padding).pack('c*')
			}
			sum_io << Common.dword_little_endian(sum_io.checksum).pack('c*')
		end

		##
		# Write this medium out to the supplied IO object, as a TRD disk image.
		# If cutoff is set to true, omits the empty sectors at the end of the disk.

		def write_trd(io, cutoff = false)
			writable_blocks = []
			sector = 16
			@blocks.each { |block|
				if block.respond_to?(:trdos_block)
					trdos_block = block.trdos_block
					sector += trdos_block.sector_count
					if sector > 2560
						$stderr << "Warning: No space on disk for block #{block}; skipping all subsequent blocks\n"
						break
					end
					writable_blocks << trdos_block
				else
					$stderr << "Warning: Cannot write block #{block} to TRD format - skipping\n"
				end
			}
			# write directory listing
			sector = 16
			0.upto(0x8d) { |i|
				if (writable_blocks[i].nil?)
					io << ([0] * 16).pack('c*')
				else
					block = writable_blocks[i]
					sector_count = block.sector_count
					io << [block.filename, block.extension, block.param1, block.param2, sector_count, sector & 0x0f, sector >> 4].pack('a8avvccc')
					sector += sector_count
				end
			}
			# write disk stats
			io << [0, sector & 0x0f, sector >> 4, 0x16, writable_blocks.size, 2560 - sector, 0x10, 0, "          ", 0, 0, @label.ljust(8), 0, 0, 0].pack('cccccvcca10cca8ccc')
			# write empty sectors 9-15
			io << ([0] * 0x700).pack('c*')
			
			# write data
			sector = 16
			writable_blocks.each { |block|
				data = block.data.to_a
				sector_count = block.sector_count
				padding = [0] * (sector_count * 256 - data.size)
				io << (data + padding).pack('c*')
				sector += sector_count
			}
			sector.upto(2559) {
				io << ([0] * 256).pack('c*')
			} unless cutoff
		end

		##
		# Convert and combine this medium's blocks into the highest-level representation
		# possible; for example, creating a medium containing two [StandardSpeedTapeBlock]s
		# that correspond to a Program: tape header and a block of BASIC data, and
		# invoking this will result in the medium containing a single BASICBlock
		
		def consolidate
			# to start, promote individual blocks as high as possible
			@blocks.collect! { |block|
				block.rise
			}
			
			# try to pair up HeaderTapeBlocks with subsequent DataTapeBlocks
			# (possibly with non-destructive blocks, such as PauseTapeBlocks, in between)
			unmatched_header = nil
			new_blocks = []
			block_queue = []
			@blocks.each { |block|
				if unmatched_header.nil?
					# no header encountered yet; pass through anything that isn't a recognised header
					if ((block.is_a? ProgramHeaderTapeBlock) || (block.is_a? BytesHeaderTapeBlock) || (block.is_a? ArrayHeaderTapeBlock))
						unmatched_header = block
						block_queue << block
					else
						new_blocks << block
					end
				else
					# we're looking for a DataTapeBlock that matches the current unmatched header
					if (block.is_a? DataTapeBlock) && (block.data.size == unmatched_header.block_length)
						# match is found - make it into an appropriate media-independent Block
						block_queue << block
						if unmatched_header.is_a? ProgramHeaderTapeBlock
							new_blocks << BASICBlock.from_tape(block_queue)
						elsif unmatched_header.is_a? BytesHeaderTapeBlock
							new_blocks << CodeBlock.from_tape(block_queue)
						elsif unmatched_header.is_a? CharacterArrayHeaderTapeBlock
							new_blocks << CharacterArrayBlock.from_tape(block_queue)
						elsif unmatched_header.is_a? NumericArrayHeaderTapeBlock
							new_blocks << NumericArrayBlock.from_tape(block_queue)
						end
						block_queue = []
						unmatched_header = nil
					elsif block.preserves_tape_loading_continuity?
						# no match, but this block can safely go between a header and its data
						block_queue << block
					else
						# a non-matching block has broken the loading continuity; give up looking for a
						# match for this header and flush the queue 
						new_blocks.concat(block_queue)
						block_queue.clear
						unmatched_header = nil
					end
				end
			}
			@blocks = new_blocks + block_queue
			
			# run a final pass of promotion in case the tape block pairing has
			# made any new promotions possible
			@blocks.collect! { |block|
				block.rise
			}

			self
		end

	end # Medium
end # ZXMedia
