#	ZXMedia: ZX Spectrum file format manipulation library
#	Copyright (C) 2004 Matthew Westcott
#	
#	This file is part of ZXMedia.
#	
#	ZXMedia is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License version 2, as
#	published by the Free Software Foundation.
#	
#	ZXMedia is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with ZXMedia; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Contact address: Matthew Westcott, 14 Daisy Hill Drive, Adlington,
#	Chorley, Lancs. PR6 9NE, United Kingdom

module ZXMedia
	class HeaderTapeBlock < Block
		# This class represents all varieties of 17-byte tape header block.
		# (Actually, by suitably misusing the extra_data field, anything from 13 bytes upward is possible)
		
		attr_reader(:datasource, :rom_tape_block)
		attr_writer(:datasource, :rom_tape_block)

		def initialize(datasource_or_header_type_id, filename = nil, block_length = nil, extra_data = nil)
			if datasource_or_header_type_id.is_a? Numeric
				# header type id supplied - other fields are required
				raise "filename not specified for HeaderTapeBlock" if filename.nil?
				raise "block_length not specified for HeaderTapeBlock" if block_length.nil?
				raise "extra_data not specified for HeaderTapeBlock" if extra_data.nil?
				@datasource = nil
				self.header_type_id = datasource_or_header_type_id
				self.filename = filename
				self.block_length = block_length
				self.extra_data = extra_data
			else
				# datasource supplied
				@datasource = datasource_or_header_type_id
			end
			self.rom_tape_block = ROMTapeBlock.new(self, 0)
		end

		def filename
			if @datasource.nil?
				@filename
			else
				@datasource.filename
			end
		end

		def filename=(f)
			raise "Cannot assign filename to this block directly" unless @datasource.nil?
			# trim / space-pad filename to 10 characters
			@filename = f.ljust(10)[0,10]
		end

		def block_length
			if @datasource.nil?
				@block_length
			else
				@datasource.block_length
			end
		end

		def block_length=(block_length)
			raise "Cannot assign block_length to this block directly" unless @datasource.nil?
			@block_length = block_length
		end

		def header_type_id
			if @datasource.nil?
				@header_type_id
			else
				@datasource.header_type_id
			end
		end

		def header_type_id=(header_type_id)
			raise "Cannot assign header_type_id to this block directly" unless @datasource.nil?
			@header_type_id = header_type_id
		end

		def extra_data
			if @datasource.nil?
				@extra_data
			else
				@datasource.header_tape_extra_data
			end
		end

		def extra_data=(extra_data)
			raise "Cannot assign extra_data to this block directly" unless @datasource.nil?
			@extra_data = extra_data
		end

		def rom_tape_data
			[self.header_type_id] + self.filename.ljust(10)[0,10].unpack('C*') + Common.word_little_endian(self.block_length) + self.extra_data
		end
		
		def to_s
			"Header type #{self.header_type_id} for file \"#{self.filename}\""
		end

		def write_tap_block(io)
			self.rom_tape_block.write_tap_block(io)
		end
		
		def write_tzx_block(io)
			self.rom_tape_block.write_tzx_block(io)
		end

		def HeaderTapeBlock.can_promote?(block)
			if block.is_a? ROMTapeBlock
				block.block_identifier == 0 && block.data.size == 17
			else
				false
			end
		end

		def HeaderTapeBlock.promote(block)
			data = block.data
			header_type_id = data[0]
			filename = data[1..10].pack('c*')
			block_length = data[11] | (data[12] << 8)
			extra_data = data[13..16]
			htb = HeaderTapeBlock.new(header_type_id, filename, block_length, extra_data)
			block.datasource = htb
			htb.rom_tape_block = block
			htb
		end

		def HeaderTapeBlock.promotion_targets
			[ProgramHeaderTapeBlock, NumericArrayHeaderTapeBlock, CharacterArrayHeaderTapeBlock, BytesHeaderTapeBlock]
		end

	end # HeaderTapeBlock
end # ZXMedia