#	ZXMedia: ZX Spectrum file format manipulation library
#	Copyright (C) 2004 Matthew Westcott
#	
#	This file is part of ZXMedia.
#	
#	ZXMedia is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License version 2, as
#	published by the Free Software Foundation.
#	
#	ZXMedia is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with ZXMedia; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Contact address: Matthew Westcott, 14 Daisy Hill Drive, Adlington,
#	Chorley, Lancs. PR6 9NE, United Kingdom

module ZXMedia
	class ROMTapeBlock < Block
		# This is a higher-level representation of a StandardSpeedTapeBlock
		# which understands the semantics of the tape load/save ROM routines;
		# i.e. the first byte is a block type flag and the last byte is a
		# checksum.

		attr_reader(:block_identifier, :checksum, :override_checksum, :datasource, :standard_speed_tape_block)
		attr_writer(:block_identifier, :checksum, :override_checksum, :datasource, :standard_speed_tape_block)

		def initialize(data, block_identifier, checksum = nil)
			# allow passing either a data array, or an external object to use as a datasource
			if data.is_a? Array
				@data = data
				@datasource = nil
			else
				@datasource = data
				@data = []
			end

			# internal StandardSpeedTapeBlock to use as a delegate
			self.standard_speed_tape_block = StandardSpeedTapeBlock.new(self)

			@block_identifier = block_identifier
			@checksum = checksum
			@override_checksum = false

			correct_check = self.correct_checksum
			if @checksum.nil?
				@checksum = correct_check
			elsif (@checksum != correct_check)
				$stderr << "Warning: checksum on tape block is #{@checksum}, but should be #{correct_check}.\n\tAutomatically fixing; set override_checksum = true to preserve the original value.\n"
			end
		end
		
		def data
			if @datasource.nil?
				@data
			else
				@datasource.rom_tape_data
			end
		end

		def data=(x)
			raise "Cannot assign data to this block directly" unless @datasource.nil?
			@data = x
		end

		def standard_speed_tape_data
			[@block_identifier] + self.data + [@override_checksum ? @checksum : self.correct_checksum]
		end

		def correct_checksum
			sum = @block_identifier
			self.data.each { |i| sum ^= i }
			sum
		end

		def to_s
			"ROM block type #{@block_identifier} (#{self.data.size} bytes)"
		end
		
		def write_tap_block(io)
			self.standard_speed_tape_block.write_tap_block(io)
		end
		
		def write_tzx_block(io)
			self.standard_speed_tape_block.write_tzx_block(io)
		end

		def ROMTapeBlock.can_promote?(block)
			(block.is_a? StandardSpeedTapeBlock) && block.data.size >= 2
		end

		def ROMTapeBlock.promote(block)
			data = block.data
			block_identifier = data.shift
			checksum = data.pop
			rtb = ROMTapeBlock.new(data, block_identifier, checksum)
			block.datasource = rtb
			rtb.standard_speed_tape_block = block
			rtb
		end

		def ROMTapeBlock.promotion_targets
			[HeaderTapeBlock, DataTapeBlock]
		end

	end # ROMTapeBlock
end # ZXMedia