#	ZXMedia: ZX Spectrum file format manipulation library
#	Copyright (C) 2004 Matthew Westcott
#	
#	This file is part of ZXMedia.
#	
#	ZXMedia is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License version 2, as
#	published by the Free Software Foundation.
#	
#	ZXMedia is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with ZXMedia; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Contact address: Matthew Westcott, 14 Daisy Hill Drive, Adlington,
#	Chorley, Lancs. PR6 9NE, United Kingdom

module ZXMedia
	class TRDOSBlock < Block

		# This class represents any file on a TR-DOS (Betadisk) filesystem.

		attr_reader :datasource
		attr_writer :datasource

		def initialize(data, filename = nil, extension = nil, param1 = nil, param2 = nil)
			# data can be either an array or an external object to use as a datasource
			if data.is_a? Array
				# array supplied - need other fields to be supplied too
				raise "filename not specified for TRDOSBlock" if filename.nil?
				raise "extension not specified for TRDOSBlock" if extension.nil?
				raise "param1 not specified for TRDOSBlock" if param1.nil?
				raise "param2 not specified for TRDOSBlock" if param2.nil?

				@data = data
				@datasource = nil
				self.filename = filename
				self.extension = extension
				self.param1 = param1
				self.param2 = param2
			else
				@datasource = data
				@data = []
			end

		end

		def filename
			if @datasource.nil?
				@filename
			else
				@datasource.filename.ljust(8)[0,8]
			end
		end

		def filename=(name)
			raise "Cannot assign filename to this block directly" unless @datasource.nil?
			@filename = name.ljust(8)[0,8]
		end

		def extension
			if @datasource.nil?
				@extension
			else
				@datasource.extension[0,1]
			end
		end

		def extension=(e)
			raise "Cannot assign extension to this block directly" unless @datasource.nil?
			@extension = e[0,1]
		end

		def data
			if @datasource.nil?
				@data
			else
				@datasource.trdos_block_data
			end
		end

		def data=(x)
			raise "Cannot assign data to this block directly" unless @datasource.nil?
			@data = x
		end

		def param1
			if @datasource.nil?
				@param1
			else
				@datasource.trdos_param1
			end
		end

		def param1=(x)
			raise "Cannot assign param1 to this block directly" unless @datasource.nil?
			@param1 = x
		end

		def param2
			if @datasource.nil?
				@param2
			else
				@datasource.trdos_param2
			end
		end

		def param2=(x)
			raise "Cannot assign param2 to this block directly" unless @datasource.nil?
			@param2 = x
		end

		def to_s
			"TR-DOS file \"#{self.filename}.#{self.extension}\" (#{self.data.size} bytes)"
		end

		def sector_count
			(self.data.size / 256.0).ceil
		end

		def write_hobeta_file(io)
			sector_count = self.sector_count
			header = (self.filename + self.extension).unpack('C*') + Common.word_little_endian(self.param1) + Common.word_little_endian(self.param2) + [sector_count, 0]


			# TODO: Check that this checksum algorithm is correct -
			# assuming there are any tools which actually look at it...
			# The code below is as literally specified in the comp.sys.sinclair FAQ,
			# which is clearly ridiculous - neither the *256 nor the +i add any 
			# integrity to the sum. (header[i] * (257 + i)) perhaps?
			gratuitous_checksum = 0
			header.each_index { |i|
				gratuitous_checksum += (header[i] * 257) + i
			}

			header += Common.word_little_endian(gratuitous_checksum & 0xFFFF)
			data = self.data.to_a
			padding = [0] * (sector_count * 256 - data.size)
			io << (header + data + padding).pack('c*')
		end

		def TRDOSBlock.promotion_targets
			[TRDOSBASICBlock, TRDOSCodeBlock]
		end

		def trdos_block
			self
		end

	end # TRDOSBlock
end # ZXMedia