#	ZXMedia: ZX Spectrum file format manipulation library
#	Copyright (C) 2004 Matthew Westcott
#	
#	This file is part of ZXMedia.
#	
#	ZXMedia is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License version 2, as
#	published by the Free Software Foundation.
#	
#	ZXMedia is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with ZXMedia; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Contact address: Matthew Westcott, 14 Daisy Hill Drive, Adlington,
#	Chorley, Lancs. PR6 9NE, United Kingdom

module ZXMedia
	class TRDOSCodeBlock < Block
		# This class represents a Code (extension C) file on a TR-DOS (Betadisk) filesystem.

		attr_reader(:shadow_data, :datasource, :trdos_block)
		attr_writer(:shadow_data, :datasource, :trdos_block)
		
		def initialize(data, filename = nil, start_address = nil, shadow_data = [])
			# data can be either an array or an external object to use as a datasource
			if data.is_a? Array
				# array supplied - need other fields to be supplied too
				raise "filename not specified for TRDOSCodeBlock" if filename.nil?
				raise "start_address not specified for TRDOSCodeBlock" if start_address.nil?

				@data = data
				@datasource = nil
				self.filename = filename
				self.start_address = start_address
			else
				@data = []
				@datasource = data
			end
			@shadow_data = shadow_data
			@trdos_block = TRDOSBlock.new(self)
		end

		def filename
			if @datasource.nil?
				@filename
			else
				@datasource.filename.ljust(8)[0,8]
			end
		end

		def filename=(name)
			raise "Cannot assign filename to this block directly" unless @datasource.nil?
			@filename = name.ljust(8)[0,8]
		end

		def data
			if @datasource.nil?
				@data
			else
				@datasource.data
			end
		end

		def data=(x)
			raise "Cannot assign data to this block directly" unless @datasource.nil?
			@data = x
		end

		def extension
			"C"
		end

		def start_address
			if @datasource.nil?
				@start_address
			else
				@datasource.start_address
			end
		end
		
		def start_address=(x)
			raise "Cannot assign start_address to this block directly" unless @datasource.nil?
			@start_address = x
		end

		def trdos_param1
			self.start_address
		end

		def trdos_param2
			self.data.size
		end

		def trdos_block_data
			saved_data = self.data
			saved_sector_count = (saved_data.size / 256.0).ceil
			saved_padding = [0] * (saved_sector_count * 256 - saved_data.size)
			shadow_part = self.shadow_data.to_a
			shadow_sector_count = (shadow_part.size / 256.0).ceil
			shadow_padding = [0] * (shadow_sector_count * 256 - shadow_part.size)
			saved_data + saved_padding + shadow_part + shadow_padding
		end

		def to_s
			"TR-DOS file \"#{self.filename}.C\" CODE #{self.start_address},#{self.data.size}"
		end

		def TRDOSCodeBlock.can_promote?(block)
			if block.is_a? TRDOSBlock
				block.extension == 'C'
			else
				false
			end
		end

		def TRDOSCodeBlock.promote(block)
			start_address = block.param1
			visible_length = block.param2
			data = block.data
			visible_data = data[0, visible_length]
			shadow_data = data[visible_length..(data.size - 1)]
			tcb = TRDOSCodeBlock.new(visible_data, block.filename, start_address, shadow_data)
			block.datasource = tcb
			tcb.trdos_block = block
			tcb
		end

		def TRDOSCodeBlock.promotion_targets
			[CodeBlock]
		end

	end # TRDOSCodeBlock
end # ZXMedia